﻿// --------------------------------------------------------------------------------
// <copyright file="TypeBasedTemplateSelector.cs" company="Interact Software Ltd.">
// © 2014 Interact Software Ltd.
// </copyright>
// --------------------------------------------------------------------------------

namespace TemplateSwitching
{
    using System.Collections.Generic;
    using Windows.UI.Xaml;
    using Windows.UI.Xaml.Controls;
    using Windows.UI.Xaml.Markup;

    /// <summary>
    /// Template selector for choosing between templates based on item types in
    /// a heterogeneous list. 
    /// </summary>
    /// <remarks>
    /// <para>
    /// We use this to show group headers and items in a list in which a set
    /// of groups has been flattened into a single list. WinRT's
    /// <c>ListView</c> does not support more than a single level of grouping. And
    /// if you attempt to nest one <c>ListView</c> inside another, their deferred update
    /// causes them to interact badly, and it seems impossible to get a stable layout.
    /// So if you want to nest groups within groups, the only practical way to do this
    /// is to flatten the inner groups and their items into a single list, so that
    /// the <c>ListView</c> only sees the outer groups. The list contains a mixture
    /// of view models representing items, and view models representing nested group
    /// headers.
    /// </para>
    /// <para>
    /// To present the inner groups correctly, we need to be able to use different
    /// templates for nested group headers and the items in those nested groups.
    /// WinRT doesn't currently support data template selection based on item type,
    /// which is how we'd deal with this in WPF. So instead, we need this template
    /// selector to do the job we'd normally expect the XAML framework to do for us.
    /// </para>
    /// </remarks>
    [ContentProperty(Name = "Templates")]
    public class TypeBasedTemplateSelector : DataTemplateSelector
    {
        private readonly Dictionary<string, DataTemplate> _templates =
            new Dictionary<string, DataTemplate>();

        /// <summary>
        /// Gets a dictionary in which templates can be associated with type names.
        /// </summary>
        /// <summary>
        /// For each item, this selector will look for an entry in this dictionary
        /// in which the key matches the name of the item type.
        /// </summary>
        public Dictionary<string, DataTemplate> Templates
        {
            get { return _templates; }
        }

        /// <summary>
        /// Gets or sets the data template to use for items whose types do not match
        /// any of the type names in the <see cref="Templates"/> dictionary.
        /// </summary>
        public DataTemplate DefaultTemplate { get; set; }

        /// <inheritdoc/>
        protected override DataTemplate SelectTemplateCore(object item)
        {
            DataTemplate template;
            if (item == null || !Templates.TryGetValue(item.GetType().Name, out template))
            {
                template = DefaultTemplate;
            }

            return template;
        }
    }
}
